<?php 
namespace Api\Controllers;
use App\Controllers\BaseController;
include_once("I18n.php");
use I18n;

class Apilogin extends BaseController {
    /**
     * Constructor.
     *
     */
    public function __construct() {
        
    }

    /**
     * Index Page for this controller and slimmed down API-login page to the GUI.
     * 
     * Note: This doesn't show the login form. It accepts POST requests of UN/PW/URL
     * if the API is enabled. Otherwise it logs errors and redirects to /login.
     */

    // XSS cleaner. 
    //
    // Please note: The regexp are taken from basetypes.schema for the corresponding inputs and conform 
    // with what CODB would accept in the fields 'username' and 'password'.
    public function xssafeLogin($data, $encoding='UTF-8', $type='') {
        if ($type == 'username_field') {
            if (!preg_match('/^[A-Za-z0-9\._-]+$/', $data)) {
                $data = '';
                return $data;
            }
            else {
                return $data;
            }
        }
        elseif ($type == 'password_field') {
            if (!preg_match('/^[^\001-\037\042\177]{6,32}$/', $data)) {
                $data = '';
                return $data;
            }
            else {
                return $data;
            }
        }
        else {
            return htmlspecialchars($data,ENT_QUOTES | ENT_HTML401,$encoding);
        }
    }

    public function index() {

        $CI =& get_instance();

        if ($this->request->getMethod() == 'post') {
            bx_error_log("Apilogin.php: login() with POST data.");
        }
        else {
            bx_error_log("Apilogin.php: login() without POST data.");
        }

        // Set up I18n:
        $i18n = new I18n("base-alpine", $CI->getBX_Locale());
        $System = $CI->getSystem();

        $servername = $System['hostname'] . '.' . $System['domainname'];

        // Strip out the :444 or :81 from the hostname - if present:
        if (preg_match('/:/', $servername)) {
            $hn_pieces = explode(":", $servername);
            $servername = $hn_pieces[0];
        }

        // Get URI string:
        $request = \Config\Services::request();
        $get_uri_string = (string) $request->getPath();

        // URI string extraction:
        $uri_elements = mb_split("\/", $get_uri_string);

        $form_data = $this->request->getPost();
        $get_form_data = $this->request->getGet();

        // locale and charset setup:
        $ini_langs = initialize_languages(TRUE);
        $locale = $ini_langs['locale'];
        $charset = $ini_langs['charset'];

        // Set cookie for locale if we do NOT have one yet. This HAS to be done here, as the entire
        // i18n she-bang heavily depends on it. 
        setcookie("locale", $locale, "0", "/");

        // Get the IP address of the user accessing the GUI:
        $userip = $_SERVER["REMOTE_ADDR"];

        if (!isset($i18n)) {
            $i18n = new I18n("base-alpine", $locale);
        }

        // If we have form data, we sanitize it:
        $attributes = array();
        $ignore_attributes = array();
        if ($this->request->getPost(NULL, NULL, TRUE)) {
            foreach ($form_data as $key => $value) {
                // Sanitize data received via form fields:
                $form_data[$key] = Apilogin::xssafeLogin($value, 'UTF-8', $key);
            }
            $required_keys = array('username_field', 'password_field', 'secureConnect', 'redirect_target');
            $attributes = GetFormAttributes($i18n, $form_data, $required_keys, $ignore_attributes);
            $form_data = $attributes;
        }
        else {
            // Not a POST request. Redirect to /login instead:
            error_log("FAILED API GET ACCESS: IP address: " . $userip);
            delete_cookie("sessionId");
            header('Location: /login');
            exit;
        }

        // Get 'System' object
        $System = $CI->cceClient->getObject('System');
        $API = $CI->cceClient->get($System['OID'], 'API');

        // Is API access allowed?
        $api_access = '0';
        if (isset($API['enabled'])) {
            if ($API['enabled'] == '1') {
                $api_access = '1';
            }
        }
        if ($api_access == '0') {
            // Not a POST request. Redirect to /login instead:
            error_log("ACCESS AGAINST DISABLED API FROM: IP address: " . $userip);
            delete_cookie("sessionId");
            $form_data = array();
            header('Location: /login');
            exit;
        }

        if ((!isset($form_data['username_field'])) || (!isset($form_data['password_field']))) {
            // No login data posted. Redirect to /login instead:
            error_log("FAILED API POST ACCESS: " . json_encode($form_data) . " - IP address: " . $userip);
            delete_cookie("sessionId");
            header('Location: /login');
            exit;
        }

        //
        //-- Form data has been sanitized and validated. Now we check if it matches:
        //

        // Get session ID
        $sessionId = $CI->cceClient->auth($form_data['username_field'], $form_data['password_field']);

        $CI->cceClient->authkey($form_data['username_field'], $sessionId);

        // Get 'System' object
        $System = $CI->cceClient->getObject('System');

        // auth failed?
        if ($sessionId == "") {
            // Login failed. Redirect to /login instead:
            error_log("FAILED API LOGIN: username: " . $form_data['username_field'] . " - IP address: " . $userip);
            delete_cookie("sessionId");
            header('Location: /login');
            exit;
        }
        else {

            // Now if this user is known (and at this point he is), then we check his CODB object to 
            // learn which locale (and charset) he's usually using and use that one instead. After all,
            // it might be different from what his browser tells us:
            $user = $CI->cceClient->getObject("User", array("name" => $form_data['username_field']));

            // Now set the locale based on the users localePreference - if specified and known:
            if ($user['localePreference']) {
                $locale = $user['localePreference'];
            }

            // Cleanup Cache:
            $cap_file_name = "/usr/sausalito/capcache/" . $form_data['username_field'] . "_cap";
            $capabilityGroups_file_name = "/usr/sausalito/capcache/" . $form_data['username_field'] . "_capabilityGroups";
            if (is_file($cap_file_name)) {
              system("rm -f $cap_file_name");
            }
            if (is_file($capabilityGroups_file_name)) {
              system("rm -f $capabilityGroups_file_name");
            }

            // Securely handle 'tzoff' for CSRF timezone offsets. See /usr/sausalito/ui/chorizo/ci/system/core/Security.php for more info.
            $tzoff_file_name = "/usr/sausalito/capcache/" . $form_data['username_field'] . '_' . $sessionId . "_tzoff";
            if (isset($_COOKIE['tzoff'])) {
                if (write_file($tzoff_file_name, $_COOKIE['tzoff'])) {
                  delete_cookie("tzoff");
                }
            }

            setcookie("loginName", $form_data['username_field'], time()+60*60*24*365, "/");
            setcookie("sessionId", $sessionId, "0", "/");
            setcookie("userip", $userip, "0", "/");
            setcookie("locale", $locale, "0", "/");

            // Manually set CSRF cookie:
            //setcookie($BX_SESSION['csrf_cookie_name'], $csrf['hash'], "0", "/");

            //
            //-- Start: Chorizo's Style handling:
            //
  
            // Read the Chorizo's Style from User's CODB object:
            if (isset($user['ChorizoStyle'])) {
                $usersChorizoStyleObject = json_decode(urldecode($user['ChorizoStyle']));
                // Turn Style Object into an Array:
                $usersChorizoStyle = (array) $usersChorizoStyleObject;
            }
            else {
                $usersChorizoStyle = array(
                                          'theme_switcher_php-style' => 'theme_blue.css',
                                          'layout_switcher_php-style' => 'layout_fixed.css',
                                          'nav_switcher_php-style' => 'switcher.css',
                                          'skin_switcher_php-style' => 'skin_light.css',
                                          'bg_switcher_php-style' => 'switcher.css'
                                          );
            }
            // If the user uses a mobile device, we override the fixed layout and switch
            // to the fluid one for a better user-experience:
            if ($this->agent->isMobile()) {
                $usersChorizoStyle['layout_switcher_php-style'] = 'layout_fluid.css';
            }
            // Push out cookies for the Users known Style:
            foreach ($usersChorizoStyle as $key => $value) {
                  setcookie($key, $value, "0", "/");
            }

            //
            //-- Redirect to where we're supposed to go:
            //

            if (isset($form_data['redirect_target'])) {
                header('Location: ' . $form_data['redirect_target']);
                exit;
            }
            else {
                // Redirect to GUI:
                header('Location: /gui');
                exit;
            }
        }
    }
}

/*
Copyright (c) 2008-2022 Michael Stauber, SOLARSPEED.NET
Copyright (c) 2008-2022 Team BlueOnyx, BLUEONYX.IT
All Rights Reserved.

1. Redistributions of source code must retain the above copyright 
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright 
   notice, this list of conditions and the following disclaimer in 
   the documentation and/or other materials provided with the 
   distribution.

3. Neither the name of the copyright holder nor the names of its 
   contributors may be used to endorse or promote products derived 
   from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.

You acknowledge that this software is not designed or intended for 
use in the design, construction, operation or maintenance of any 
nuclear facility.

*/
?>